/************************************************************************ *********
 * Proiectarea cu Microprocesoare
 * Laboratorul 1
 * Digital I/O
 *********************************************************************************/

#include <avr\io.h>
#include <avr\interrupt.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
//#include <util/delay.h>

/************************************************************************ 
 * DEFINE-uri pentru parametrizarea modulului LCD
 ************************************************************************/
#define LcdDATA_DDR		DDRD			// Portul pe care conectam firele de date la LCD-ul
#define LcdDATA_PORT    PORTD
#define LcdDATA_PIN		PIND

#define LcdCMD_DDR		DDRD			// Portul pe care conectam firele de comenzi la LCD
#define LcdCMD_PORT		PORTD
#define LcdCMD_PIN		PIND

#define LcdD4			 PD0				// Pin-ul pentru firul de date D4 de pe LCD
#define LcdD5			 PD1				// Pin-ul pentru firul de date D5 de pe LCD
#define LcdD6			 PD2				// Pin-ul pentru firul de date D6 de pe LCD
#define LcdD7			 PD3				// Pin-ul pentru firul de date D7 de pe LCD

#define LcdRS			 PD4			    // Pinul pentru selectare operatie (LCD)
#define LcdRW			 PD5				// Pinul pentru Read/ Write (LCD)
#define LcdE			 PD6				// Pinul de Enable (LCD)

#define LCD_INSTR_4wire 		0x28 	// 4 fire date, font 5x8
#define LCD_INSTR_display 		0x0C 	// Display On, Cursor On, Blinking On ( 1 Display Cursor Blink )
#define LCD_INSTR_clearDisplay  	0x01 	// Clear Display
#define LCD_INSTR_returnHome		0x02 	// Return Cursor and LCD to Home Position
#define LCD_INSTR_nextLine 		0xC0 	// Return Cursor and LCD to Home Position
#define LCD_INSTR_gotoCGRAM		0x40	// go to Character Generator RAM


// fst
#define RS_HIGH()				LcdCMD_PORT |= _BV(LcdRS)
#define RS_LOW()				LcdCMD_PORT &= ~_BV(LcdRS)

#define RW_HIGH()				LcdCMD_PORT |= _BV(LcdRW)
#define RW_LOW()				LcdCMD_PORT &= ~_BV(LcdRW)

#define ENABLE()				LcdCMD_PORT |= _BV(LcdE)
#define DISABLE()				LcdCMD_PORT &= ~_BV(LcdE)

#define Buton      PD6
#define Buton_PORT PORTD
#define Buton_PIN  PIND
#define Buton_DDR  DDRD

#define LedBoard      PD7
#define LedBoard_PORT PORTD

// variabile globale

char FAZAN[6]={'F','A','Z','A','N','\0'};
char hexa[16]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
char alfabet[26]={'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z'};
char litera[32]={'A','E','D','P','M','C','N','L','B','H','I','G','F','J','O','K','Q','U','T','!',',','S','.','-','R','X','Y','W','V','Z','?',' '};
char dictionar[10][11]={
{'A','A','L','E','N','I','A','N','\0'},
{'A','B','A','\0'},
{'A','B','A','C','\0'},
{'A','B','A','C','A','\0'},
{'A','B','A','G','E','R','\0'},
{'A','B','A','G','I','U','\0'},
{'A','B','A','J','U','R','\0'},
{'A','B','A','N','O','S','\0'},
{'R','A','G','L','A','N'},//{'A','B','A','N','D','O','N','A','R','E','\0'},
{'A','N','C','O','R','A','\0'}//{'A','B','A','N','O','S','\0'}
};

/*************
 * API LCD   *
 *************/
void LCD_init();                // Initializare modul LCD.Trebuie apelata inainte de a se face orice operatie cu LCD-ul
void LCD_writeInstruction(unsigned char _instruction);	// Trimite o instructiune catre lcd (vezi datasheet)
void LCD_writeData(unsigned char _data);		// Trimite date catre LCD pentru afisare
void LCD_write(unsigned char _byte);			// trimite un bute catre LCD la modul general (nu conteaza daca e instructiune sau date)
void LCD_waitNotBusy();					// Asteptam pana cand lcd-ul devine disponibil pt o noua comanda
void LCD_waitInstructions(unsigned char _instructions);	// Asteapta un numar de cicli de ceas.
void LCD_print(char* _msg);				// Afiseaza imformatia pe LCD (doar 1 linie, primele 16 caractere)
void LCD_print2(char* _msg1, char* _msg2);		// Afisare pe 2 lini pe LCD
void LCD_printDecimal2u(unsigned int _n);		// Afisare numar in baza 10 pe LCD
void LCD_printHexa(unsigned int _n);			// Afisare numar in baza 16 pe LCD




/*****************************
 * IMPLEMENTARE API LCD.     *
 *****************************/

void LCD_init()
{
	LcdDATA_DDR |=  (1<<LcdD4)|(1<<LcdD5)|(1<<LcdD6)|(1<<LcdD7);	// setam pinii de date ca pini de iesire
	LcdCMD_DDR  |=  (1<<LcdRS)|(1<<LcdRW)|(1<<LcdE);				// setam pinii de comenzi ca pini de iesire

	LCD_waitNotBusy();
	
	LcdCMD_PORT   &= ~(1<<LcdRS);									// Setam linia RS pe low
	LcdCMD_PORT   &= ~(1<<LcdRW);									// Setam linia RW pe low (acum suntem in modul de trimis instructiuni)
	LcdDATA_PORT  &= ~(1<<LcdD4)&~(1<<LcdD6)&~(1<<LcdD7); 		// Specificam ca vrem 4 fire de date, prima comanda (LcdD5 activ, restul nu)
	LcdDATA_PORT  |=  (1<<LcdD5);									// setam pinii de comenzi ca pini de iesire
	
	LcdCMD_PORT |=  (1<<LcdE);					// Setam linia E(nable) pe high; aceasta ii specifica LCD-ului sa preia datele
	LCD_waitInstructions(6);					// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE );				// Setam linia E(nable) pe low; transferul s-a terminat

	LCD_writeInstruction(LCD_INSTR_4wire); 		// Incarcam comanda: 4 bit data, 2 lines, 5x8 font
	LCD_writeInstruction(LCD_INSTR_display); 	// Display On, Cursor On, Blinking On
	LCD_writeInstruction(0x06);					// Increment, no shift
	LCD_writeInstruction(0x01);					// Clear Display

	
}

void LCD_writeInstruction(unsigned char _instruction)
{
	/* TODO */					
	// asteptam ca LCD-ul sa fie liber sa primeasca comenzile
	LCD_waitNotBusy();
	// setam pinul RS pe low (low=instructiuni, high=date)
	LcdCMD_PORT   &= ~(1<<LcdRS);
	// setam pinul RW pe low (suntem in modul de comenzi acum)
	LcdCMD_PORT   &= ~(1<<LcdRW);
	// apelam procedura ce trimite byte-ul pe firele de date
	LCD_write(_instruction);
}


void LCD_writeData(unsigned char _data)
{
        /* TODO */
	// similar cu LCD_writeInstruction
	// asteptam ca LCD-ul sa fie liber sa primeasca comenzile
	LCD_waitNotBusy();
	// setam pinul RS pe high (low=instructiuni, high=date)
	LcdCMD_PORT   |= (1<<LcdRS);
	// setam pinul RW pe low (suntem in modul de comenzi acum)
	LcdCMD_PORT   &= ~(1<<LcdRW);
	// apelam procedura ce trimite byte-ul pe firele de date
	LCD_write(_data);
}

void LCD_write(unsigned char _byte)
{
        /* TODO */
	// scriem cei mai semnificativi 4 biti pe firele de date
  LcdDATA_PORT = (LcdDATA_PORT & 0xF0) | (_byte >> 4);
	// Setam Pinul E pe high
  ENABLE();
	// Asteptam o perioada de timp T
	LCD_waitInstructions(6);					// Asteptam o perioada de timp T	
	// Setam Pinul E pe low
  DISABLE();
	// Asteptam o perioada de timp T
	LCD_waitInstructions(6);					// Asteptam o perioada de timp T	
	// scriem cei mai putin semnificativi 4 biti pe firele de date
  LcdDATA_PORT = (LcdDATA_PORT & 0xF0) | (_byte & 0x0F);
	// Setam Pinul E pe high
  ENABLE();
	// Asteptam o perioada de timp T
	LCD_waitInstructions(6);					// Asteptam o perioada de timp T	
	// Setam Pinul E pe low
  DISABLE();
}

void LCD_waitNotBusy()
{
	unsigned char _loop = 1;

	while (_loop)
	{
		LcdDATA_DDR &= ~(1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7);	// Setam pinii de date de la LCD pe in pt a citi busy flag
		LcdDATA_PORT &= ~(1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7); 	// Dezactivam pullup resistor pentru pinii de in
		
		LcdCMD_PORT &= ~(1<<LcdE);					// Setam pin-ul e pe low; ar trebui sa fie deja low, doar ne asiguram
		LcdCMD_PORT &= ~(1<<LcdRS);					// Setam pinul RS pe low
		LcdCMD_PORT |=  (1<<LcdRW);					// Setam pinul RW pe high (acum suntem in modul de interogare busy/adr)
		
		LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
		LCD_waitInstructions(6);						// Asteptam o perioada de timp T
		_loop = LcdDATA_PIN & (1<<LcdD7);				// Citim busy flag-ul
		LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low
		
		LCD_waitInstructions(6);						// Asteptam o perioada de timp T
		
		LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
		LCD_waitInstructions(6);						// Asteptam o perioada de timp T
		LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low

		LcdDATA_DDR |= (1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7); // Setam Portul de LCD ca port de iesire la loc
	}
}

void LCD_waitInstructions(unsigned char _instructions)
{
	while (_instructions--)
		;
}


void LCD_printDecimal2u(unsigned int _n)
{
	unsigned char tmp=0;
	
	// Extragem sutele
	while(_n>=100)
		_n-=100;

	while(_n>=10){
		tmp++;
		_n-=10;
	}

	LCD_writeData(tmp+'0');
	LCD_writeData(_n+'0');
}

void LCD_printHexa(unsigned int _n)
{
	unsigned char _tmp = _n>>4;
	if (_tmp>9)
		_tmp += 'A'-10;
	else
		_tmp += '0';
	LCD_writeData( _tmp );
	_tmp = _n & 0x0F;
	if (_tmp>9)
		_tmp += 'A'-10;
	else
		_tmp += '0';
	LCD_writeData( _tmp );
}


void LCD_print(char* _msg)
{
	unsigned char i=0;
	for( ; _msg[i]!=0 && i<16; i++)
		LCD_writeData( _msg[i] );
}

void LCD_print2(char* _msg1, char* _msg2)
{
	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	LCD_print(_msg1);
	LCD_writeInstruction(LCD_INSTR_nextLine);
	LCD_print(_msg2);
}

int ADC_get(uint8_t channel)
{
	//TODO
	//setati canalul de pe care vreti sa faceti citirea
	//asteptati sa se termine o conversie
	//returnati rezultatul conversiei
	ADMUX   &= ~(1<<MUX2);
	ADMUX   &= ~(1<<MUX1);
	ADMUX   &= ~(1<<MUX0);
	ADMUX |= channel;
	ADCSRA   |= (1<<ADSC);
	while ((ADCSRA & (1<<ADIF)) == 0) ;
	return ADC;
}

void ADC_init(void)
{

	ADCSRA   |= (1<<ADEN);
	ADCSRA   |= (1<<ADPS0);
	ADCSRA   |= (1<<ADPS1);
	ADCSRA   |= (1<<ADPS2);
	ADCSRA   |= (1<<ADSC);
	ADMUX   &= ~(1<<REFS1);
	ADMUX   |= (1<<REFS0);
}

int main()
{
	int valUnu,valDoi,valTrei,valSase,valSapte,i=0,buton,ambiental,led_aprins,p,gasit, vieti_jucator=5,vieti_AI=5;
	char *s,*ss,cuvant[15],litera_curenta,temp[14];

  // initializam LCD-ul
	LCD_init(); 
  
  // initializam convertorul analog-digital
	ADC_init();
  
	// Alocam memorie pentru cele doua randuri de afisare
  s=malloc(16);
	ss=malloc(16);
  
  // scriem un mesaj la inceputul jocului
	LCD_print("**** Fazan! ****");
  
  // Loading... --> aici trebuie citit de pe cardul SD dictionarul si creat arborele
  for (i=0; i<1500; i++);
  
	cuvant[0]='\0';
  
  // activam rezistenta de pull-up pt butonul de pe placuta
	Buton_DDR &= ~(1<<Buton); //fst
	Buton_PORT |= (1<<Buton); //fst
  
  // afisam un meniu initial
  LCD_writeInstruction(LCD_INSTR_clearDisplay);
  LCD_print2("Pentru START","apasati s");
  
	for(p=0;p<1500;p++);
  
	valUnu = ADC_get(1);
	valDoi = ADC_get(2);
	valTrei = ADC_get(3);
	valSase = ADC_get(6);
	valSapte = ADC_get(7);
	ambiental = valUnu + 100;
	led_aprins = (valDoi+valTrei+valSase+valSapte)/4 + 9;
	
	while (vieti_jucator && vieti_AI) {
	
		while(litera_curenta!=',') {
		
			for(p=0;p<1000;p++);
			
			buton=0;
			if((Buton_PIN & (1<< Buton))==0)
				buton=1;
			
			valDoi = ADC_get(2);
			valTrei = ADC_get(3);
			valSase = ADC_get(6);
			valSapte = ADC_get(7);
			
			litera_curenta = litera[buton*16+valDoi/led_aprins*8+valTrei/led_aprins*4+valSase/led_aprins*2+valSapte/led_aprins];
			
			strcpy(temp,FAZAN);
			temp[5-vieti_jucator]='\0';
			sprintf(s, "HP: %s %c",temp,litera_curenta);
			sprintf(ss,"%d %d %d %d %d" ,valUnu,valDoi,valTrei,valSase,valSapte);
			
			LCD_writeInstruction(LCD_INSTR_clearDisplay);
			LCD_print2(s,ss);
		}

		while(litera_curenta!='.'){

			for(p=0;p<1000;p++);
			
			buton=0;
			if((Buton_PIN & (1<< Buton))==0)
				buton=1;
			
			valDoi = ADC_get(2);
			valTrei = ADC_get(3);
			valSase = ADC_get(6);
			valSapte = ADC_get(7);
			
			litera_curenta = litera[buton*16+valDoi/led_aprins*8+valTrei/led_aprins*4+valSase/led_aprins*2+valSapte/led_aprins];

			strcpy(temp,FAZAN);
			temp[5-vieti_AI]='\0';
			sprintf(s, "AI: %s %c",temp,litera_curenta);
			sprintf(ss,"%d %d %d %d %d" ,valUnu,valDoi,valTrei,valSase,valSapte);

			LCD_writeInstruction(LCD_INSTR_clearDisplay);
			LCD_print2(s,ss);
		}
		i=0;
		while(litera_curenta!='!'){

			for(p=0;p<1000;p++);
			
			buton=0;
			if((Buton_PIN & (1<< Buton))==0)
				buton=1;
			
			valDoi = ADC_get(2);
			valTrei = ADC_get(3);
			valSase = ADC_get(6);
			valSapte = ADC_get(7);
			
			litera_curenta = litera[buton*16+valDoi/led_aprins*8+valTrei/led_aprins*4+valSase/led_aprins*2+valSapte/led_aprins];
			
			sprintf(s, "%c", alfabet[i++%26]);
			sprintf(ss,"%d %d %d %d %d" ,valUnu,valDoi,valTrei,valSase,valSapte);

			LCD_writeInstruction(LCD_INSTR_clearDisplay);
			LCD_print2(s,ss);
		}
		
		cuvant[0]=alfabet[(i-1)%26];
		gasit=0;
		
		while(!gasit){
			i=0;
			litera_curenta='\0';
			while(litera_curenta!='.'){
				for(p=0;p<1200;p++);
				
				buton=0;
				if((Buton_PIN & (1<< Buton))==0)
					buton=1;
				
				valUnu = ADC_get(1);
				valDoi = ADC_get(2);
				valTrei = ADC_get(3);
				valSase = ADC_get(6);
				valSapte = ADC_get(7);
				
				litera_curenta = litera[buton*16+valDoi/led_aprins*8+valTrei/led_aprins*4+valSase/led_aprins*2+valSapte/led_aprins];
				
				if(valUnu>ambiental){
					temp[i++]=litera_curenta;
					temp[i]='\0';					  
				}
				
				sprintf(s,"%c%s %c", cuvant[0], temp, litera_curenta);
				sprintf(ss,"%d %d %d %d %d" ,valUnu,valDoi,valTrei,valSase,valSapte);
		
				if(valUnu > ambiental)
					LedBoard_PORT   |= (1<<LedBoard);
				else
					LedBoard_PORT   &= ~(1<<LedBoard);
				
				LCD_writeInstruction(LCD_INSTR_clearDisplay);
				LCD_print2(s,ss);
			}
      cuvant[1]='\0';
      strcat(cuvant,temp);
      
      for(i=0;i<10;i++){
        if(strcmp(cuvant,dictionar[i])==0){
          gasit=1;
          break;
        }
      }
      if(gasit==0)
        sprintf(s,"Incorect!");
      else
        sprintf(s,"Corect!");
			LCD_writeInstruction(LCD_INSTR_clearDisplay);
			LCD_print(s);
		}
	}

	return 0;
}
